node = {
    "node_data": {
        "msg": {
            "action": "add_node",
            "nodes": [
            ]
        }
    },
    "link_data": {
        "msg": {
            "action": "add_link",
            "links": [
            ]
        }
    },
    "node": {
        "id": None,
        "attr": "Unknow",
    },
    "link": {
        "from": None,
        "to": None,
        "value": 60,
    },

    "must": [
        "id", "from", "to"
    ]

}

geo = {
    "msg": {
        "msg":{
            "action": "geo",
            "title": None,
            "content": None,
            "type": None,
            "place": None,
        }
    },
    "control":{
        "msg":{
            "action":"move_to",
            "place": None,
        }
    },
    "zoom":{
        "msg":{
            "action":"zoom",
            "zoom": None,
        }
    },
    "mark":{
        "msg":{
            "action": "mark",
            "coor": None,
            "msg": None
        }
    },
    "sky_line":{
        "msg":{
            "action": "fly_link",
            "from": None,
            "to": None,
            "time": 60,
        }
    },

}



import re
import time
from qlib.io import get_output


def text2dict(text):
    dit = {}
    for line in text.split("\n"):
        if not line.strip():
            continue

        arg = line.split()
        if len(arg) == 1:
            dit['title'] = arg[0]
            continue
        if arg[1].startswith("[") and arg[-1].endswith("]"):
            dit[arg[0]] = arg[1:]
            continue

        dit[arg[0]] = arg[1] if len(arg) == 2 else " ".join(arg[1:])
    return dit


def symbol(c):
    return str(id(c) + time.time())


def no_symbol(text, repl=' '):
    return re.sub(r'[^\w^d^\n\s]', repl, text)


def text_to_tree(dealnode, dealrealtion, sleep=None, console=False):
    l_num = [0]
    t = {0: ['r']}
    ls = []
    for l in get_output(console):
        search_res = re.search(r'\w', l)
        if not search_res:
            continue
        s_n = search_res.span()[0]

        if s_n in l_num:
            pass
        else:
            l_num.append(s_n)
            l_num.sort()
        fathre_index = l_num.index(s_n) - 1
        fathre_index = fathre_index if fathre_index >= 0 else 0
        # print(l_num[fathre_index])
        father = t[l_num[fathre_index]][-1]
        node = (symbol(l), l[s_n:])

        if s_n in t:
            t[s_n].append(node)
        else:
            t[s_n] = [node]
        ls.append((father, node, s_n))
        dealnode(node)
        if sleep:
            time.sleep(sleep)

    for link in ls:
        dealrealtion(*link)
        if sleep:
            time.sleep(sleep)


# def text_to_map(net)


import sys
from contextlib import contextmanager


def get_output(console=True):
    blank_count = 0
    while True:

        l = sys.stdin.readline().rstrip() # remove \n
        if not l:
            if blank_count > 100:
                break
            blank_count += 1
            continue
        else:
            if blank_count > 0:
                blank_count -= 1

        if console:
            print(l)
        yield l


@contextmanager
def stdout(fp):
    stdo = sys.__stdout__
    try:
        sys.stdout = fp
        yield
    finally:
        sys.stdout = stdo



import os
import sys
import atexit
import signal

from functools import partial
from termcolor import cprint, colored

def daemonize(pidfile, *, stdin='/dev/null',
              stdout='/dev/null',
              stderr='/dev/null'):
    if os.path.exists(pidfile):
            raise RuntimeError('Already running')
    # First fork (detaches from parent)
    try:
        if os.fork() > 0:
            raise SystemExit(0)
    except OSError as e:
        raise RuntimeError('fork #1 failed.')
    os.chdir('/')
    os.umask(0)
    os.setsid()
    # Second fork (relinquish,session leadership)
    try:
        if os.fork() > 0:
            raise SystemExit(0)
    except OSError as e:
        raise RuntimeError('fork #2 failed.')
    # Flush I/O buffers
    sys.stdout.flush()
    sys.stderr.flush()
    # Replace file descriptors for stdin, stdout, and stderr
    with open(stdin, 'rb', 0) as f:
        os.dup2(f.fileno(), sys.stdin.fileno())
    with open(stdout, 'ab', 0) as f:
        os.dup2(f.fileno(), sys.stdout.fileno())
    with open(stderr, 'ab', 0) as f:
        os.dup2(f.fileno(), sys.stderr.fileno())
        # Write the PID file
    with open(pidfile, 'w') as f:
        print(os.getpid(), file=f)
    # Arrange to have the PID file removed on exit/signal
    atexit.register(lambda: os.remove(pidfile))
    # Signal handler for termination(required)

    def sigterm_handler(signo, frame):
        raise SystemExit(1)
    signal.signal(signal.SIGTERM, sigterm_handler)


def test(): 
    import time
    sys.stdout.write('Daemon started with pid {}\n'.format(os.getpid()))
    while True:
        sys.stdout.write('Daemon Alive! {}\n'.format(time.ctime()))
        time.sleep(10)


def run(func):
    PID_F = os.path.join("/var/run/", func.__name__ + ".pid")
    LOG_F = os.path.join("/var/log/", func.__name__ + ".log")
    ERROR_F = os.path.join("/var/log/", func.__name__ + ".error.log")
    try:
        print(func.__name__, colored("[start]", "green", attrs=['blink']))
        daemonize(PID_F, stdout=LOG_F, stderr=ERROR_F)
    except RuntimeError as e:
        cprint(e, "red", file=sys.stderr)
        raise SystemExit(1)
    func()


def restart(func):
    print("--- restart ---")
    stop(func)
    print("--- stop %s wait to starting ---" % func.__name__)
    run(func)
    print("--- start %s ---" % func.__name__)


def stop(func):
    PID_F = os.path.join("/var/run/", func.__name__ + ".pid")
    LOG_F = os.path.join("/var/log/", func.__name__ + ".normal.log")
    for i in [PID_F]:
        if os.path.exists(i):
            with open(i) as f:
                os.kill(int(f.read()), signal.SIGTERM)
        else:
            print("Not running : ", i, file=sys.stderr)
    from time import asctime
    with open(LOG_F, "a+") as fp:
        print(asctime() ," -- stop --", file=fp)
        cprint("-- stop --", "blue")
    raise SystemExit(1)



#!/usr/bin/env python3
import logging
from functools import wraps
from urllib.parse import urljoin
from .funcs import *


class attr:

    links = "//a[@href!='#'][@href!='javascript:;'][@href!='#']"
    green = "\033[0;32m%s\033[0m"
    red = "\033[0;31m%s\033[0m"
    yellow = "\033[0;33m%s\033[0m"
    checktor = {
        " and 1=1--- and 1=2": eq,
    }

def getHeaders(host, text=None, **kargs):
    headers_2 = { i.split(':')[0]: i.split(':')[1] for i in text.split("\n")}
    raw_headers  = {
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'en-US,en;q=0.8',
        'Cache-Control': 'max-age=0',
        'Connection': 'keep-alive',
        'Host': host,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.106 Safari/537.36'
    }
    for k in headers_2:
        raw_headers[k] = headers_2[k]
        
    for k in kargs:
        raw_headers[k] = kargs[k]
    return raw_headers

def check_url(url_head, url, ssl=False):
    http_head = "https://" if ssl else "http://"
    domain = url_head if url_head.startswith("http") else http_head + "www." + url_head + ".com"
    # check first arg
    arg = url
    if arg.startswith("http"):
        if arg.find(url_head) > -1:
            return arg
        logging.warn(attr.yellow % "This link [out]" + arg)
        return None
    if arg.startswith("/"):
        return urljoin(domain, arg)
    elif arg.startswith("www"):
        if arg.find(url_head) > -1:
            return http_head + arg
        logging.warn(attr.red % "This link invalid! [out]" + arg)
        return None
    else:
        arg = urljoin(domain, arg) 
        return arg





import time
import datetime
from hashlib import sha256


def _hash(obj):
    return sha256(obj.encode('utf8')).hexdigest()



class SqlEngine:


    DEFAULT_TABLE = """CREATE TABLE %s (
        ID INTEGER PRIMARY KEY NOT NULL AUTO_INCREMENT ,
        CreatedTime TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP, 
        {extends} );"""

    def __init__(self, database=None, type='sqlite' , **connect_options):
        self.Type = type
        self.database = database
        engine = None
        if not database and type == 'sqlite':
            type = 'mysql'
            self.Type = 'mysql'
            database = 'test'

        try:
            if type == 'sqlite':
                self.DEFAULT_TABLE = self.DEFAULT_TABLE.replace('AUTO_INCREMENT', '')
                import sqlite3
                engine = sqlite3
            elif type == 'mysql':
                import pymysql
                engine = pymysql
            elif type == 'postgresql':
                self.DEFAULT_TABLE = self.DEFAULT_TABLE.replace('ID INTEGER PRIMARY KEY NOT NULL AUTO_INCREMENT', 'ID SERIAL PRIMARY KEY')
                import psycopg2
                engine = psycopg2
            else:
                raise Exception("un supported sql: %s" % type)

        except ImportError as e:
            print(e)
        if self.Type == 'sqlite':
            self.con = engine.connect(database, **connect_options)
        else:

            self.con = engine.connect(database=database, **connect_options)
        self.cu = self.con.cursor()

    def datetime(timeobj):
        if isinstance(timeobj, datetime.datetime):
            return timeobj
        elif isinstance(timeobj, str):
            ss = time.strptime(timeobj, '%Y-%m-%d %H:%M:%S')
            return datetime.datetime.fromtimestamp(time.mktime(ss))


    def create_table(self, table, *extends_columns):
        columns = ',\n\t'.join(extends_columns)
        cmd = self.DEFAULT_TABLE.format(extends=columns) % table
        self.run_cmd(cmd)
        return cmd

    def create(self, table, extend_options={}, **kargs):
        """
        this function can create table smarter:
        example:
            create('User', name=str, login_time=time, passwd=hash, own=1, number=int, id_f=Table)
            will ->  CREATE TABLE User (
                            ID INTEGER PRIMARY KEY NOT NULL [AUTO_INCREMENT] ,
                            CreatedTime TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                            name TEXT NOT NULL,
                            login_time TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                            passwd text, # hash(str),
                            own INTEGER NOT NULL DEFAULT 1 ,
                            number INTEGER,
                            id_f FOREIGN KEY REFERENCES Table(ID)
            ) 
        """

        columns = {}
        names = []
        for k in kargs:
            names.append(k)
            v = kargs[k]
            if v is int:
                columns[k] = '%s %s' % (k, 'INTEGER')
            elif v is str:
                columns[k] = '%s %s' % (k, 'TEXT')
            elif v is time:
                columns[k] = '%s %s' % (k, 'TimeStamp DEFAULT CURRENT_TIMESTAMP')
            elif isinstance(v, int):
                columns[k] = '%s %s' % (k, 'INTEGER NOT NULL DEFAULT %d' % v)
            elif isinstance(v, str):
                columns[k] = '%s %s' % (k, 'VARCHAR(255) NOT NULL DEFAULT \'%s\'' % v)
            
            elif hasattr(v, '_table'):
                columns[k] = '%s %s' % (k, 'INTEGER, FOREIGN KEY (%s) REFERENCES %s(ID)' % (k, v.__name__))
            else:
                raise TypeError("not supported Sql Type %s" % str(v))

        # extend some attribute
        for k in extend_options:
            columns[k] += extend_options[k]

        return self.create_table(table, *columns.values())
        

    def drop_table(self, table):
        cmd = 'DROP TABLE %s;' % table
        return self.run_cmd(cmd)

    def close(self):
        try:
            self.cu.close()
            self.con.close()
        except Exception:
            pass
        finally:
            pass

    def __del__(self):
        self.close()

    def _sql(self, k, v):
        if isinstance(v, int):
            return '{}={}'.format(k, v)
        elif isinstance(v, str):
            return '{}=\'{}\''.format(k, v)
        elif isinstance(v, datetime.datetime):
            return '{}=\'{}\''.format(k, v)
        elif hasattr(v, '_table'):
            if hasattr(v, "ID"):
                return str(v.ID)
            return 'NULL'
        else:
            print(k, v)
            raise Exception("what type?")

    def select(self, table, *fields, tail=' ', **condition):
        if len(fields) == 0:
            fields = ("*", )

        cmd = "select {} from %s ".format(",".join(fields)) % table 
        if condition:
            cmd += 'where ' +  ' '.join([self._sql(*i) for i in condition.items()])
        # print(cmd)
        cmd = cmd + tail + ';'
        try:
            self.cu.execute(cmd)
            for row in self.cu.fetchall():
                yield row
        except Exception as e:
            if hasattr(self.con, 'rollback'):
                self.con.rollback()
            print(cmd)
            raise e


    def first(self, table, *fields, **condition):
        if len(fields) == 0:
            fields = ("*", )
        for i in self.select(table, *fields, tail=' limit 1 ', **condition):
            return i

    def last(self, table, *fields, **condition):
        if len(fields) == 0:
            fields = ("*", )
        for i in self.select(table, *fields, tail=' order by  ID desc limit 1 ', **condition):
            return i

    def check_table(self, table):
        """
        """
        if self.Type == 'sqlite':
            for name in self.select("sqlite_master", 'name',type='table'):
                if name[0] == table:
                    sql = self.first("sqlite_master", 'sql', name=table)[0]
                    _pr = sql.find('(') + 1
                    _tr = sql.rfind(')')
                    return tuple((tuple(i.split()) for i in sql[_pr:_tr].split(",")))
            return False
        elif self.Type == 'postgresql':
            res = tuple(self.select("information_schema.columns",
                'column_name',
                'data_type',
                'column_default',
                table_name=table))
            if res:
                return res
            else:
                return False
        else:
            try:
                self.run_cmd("desc " + table)
                return self.cu.fetchall()
            except Exception as e:
                if e.args[0] == 1146:
                    return False
                else:
                    raise e

    def table_list(self):
        if self.Type == "postgresql":
            return tuple(self.select('information_schema.tables', 'table_name', table_schema='public'))
        elif self.Type == 'sqlite':
            return tuple(self.select('sqlite_master', 'name'))
        elif self.Type == 'mysql':
            return tuple(self.select('information_schema.tables', 'table_name', table_schema=self.database))
        else:
            raise Exception("not supported type")

    def _sqls(self, i):
        if isinstance(i, int):
            return str(i)
        elif isinstance(i, str):
            return i
        elif hasattr(i, '_table'):
            if hasattr(i, "ID"):
                return str(i.ID)
            return 'NULL'
        elif i is time:
            return datetime.datetime.now()
        elif isinstance(i, datetime.datetime):
            return i
        else:
            print(i)
            raise("error insert")

    def insert(self, table, insert_columns, *values, **kargs):
        values = [ self._sqls(i) for i in values ]
        sept = '?' if self.Type == 'sqlite' else '%s'
        # sept = '%s'
        values_tmp = ','.join([sept for i in range(len(values))])
        cmd = ("insert into %s ({order}) values ({values});" % table).format(order=','.join(insert_columns), values=values_tmp) 
        # print(cmd, values)
        
        self.run_cmd(cmd, *values)
        

    def update(self, table, sets, **condition):
        sept = '?' if self.Type == 'sqlite' else '%s'

        cond = ' '.join([self._sql(*i) for i in condition.items()])
        updated = ','.join([self._sql(*i) for i in sets.items()])
        cmd = '''UPDATE {table}  SET {updated}
        WHERE {condition} ;
        '''.format(table=table, updated=updated, condition=cond)

        # print(cmd)
        return self.run_cmd(cmd)

    def delete(self, table, **condition):
        cond = ''
        if condition:
            cond = 'WHERE ' + ' '.join([self._sql(*i) for i in condition.items()])
        cmd = '''DELETE FROM {table}
        {condition} ;
        '''.format(table=table, condition=cond)

        return self.run_cmd(cmd)

    def alter(self, table, **columns):
        """
        @table: table is table name
        @**columns: this like 'create' funciton. when you add a new 'k=v' will add to table column k,
                    but if v is 'None' the table will remove column from table.
            example:

                In [14]: sql.check_table("tt2")
                Out[14]: 
                 (('ID', 'INTEGER', 'PRIMARY', 'KEY', 'NOT', 'NULL'),
                 ('CreatedTime', 'TimeStamp', 'NOT', 'NULL', 'DEFAULT', 'CURRENT_TIMESTAMP'),
                 ('nn', 'VARCHAR(255)', 'NOT', 'NULL', 'DEFAULT', '"default', 'text"'))

                In [15]: sql.alter("tt2", nn=None)
                Out[15]: True

                Out[16]: 
                 (('ID', 'INTEGER', 'PRIMARY', 'KEY', 'NOT', 'NULL'),
                  ('CreatedTime', 'TimeStamp', 'NOT', 'NULL', 'DEFAULT', 'CURRENT_TIMESTAMP'))
                
                In [17]: sql.alter("tt2", nn='abc')
                Out[17]: True

                In [18]: sql.check_table("tt2")
                Out[18]: 
                (('ID', 'INTEGER', 'PRIMARY', 'KEY', 'NOT', 'NULL'),
                 ('CreatedTime', 'TimeStamp', 'NOT', 'NULL', 'DEFAULT', 'CURRENT_TIMESTAMP'),
                 ('nn', 'VARCHAR(255)', 'NOT', 'NULL', 'DEFAULT', '"abc"'))

                In [8]: sql.alter("tt2", nn=None, name='name', passwd='secret', content=str, 
                   ...: ftime=time)
                Out[8]: True

                In [9]: sql.check_table("tt2")
                Out[9]: 
                (('ID', 'INTEGER', 'PRIMARY', 'KEY', 'NOT', 'NULL'),
                 ('CreatedTime', 'TimeStamp', 'NOT', 'NULL', 'DEFAULT', 'CURRENT_TIMESTAMP'),
                 ('passwd', 'VARCHAR(255)', 'NOT', 'NULL', 'DEFAULT', '"secret"'),
                 ('ftime', 'TimeStamp', 'DEFAULT', 'NNULL'),
                 ('name', 'VARCHAR(255)', 'NOT', 'NULL', 'DEFAULT', '"name"'),
                 ('content', 'TEXT', 'DEFAULT', 'NULL'))



        """
        first_word = lambda x: x.strip().split()[0]
        names = []
        cmd = ''
        table_structure = self.check_table(table)
        sqlitedrop = False

        old_columns  = [i[0] for i in table_structure]
        drop_columns = set()
        if not table_structure:
            raise Exception("no such table")

        for k in columns:
            cmd = ''
            dtype = ''
            v = columns[k]
            if v is int:
                dtype = 'INTEGER DEFAULT NULL'
            elif v is str:
                dtype = 'TEXT DEFAULT NULL'
            elif v is time:
                dtype = 'TimeStamp DEFAULT NULL' if self.Type == 'sqlite' else 'TimeStamp DEFAULT CURRENT_TIMESTAMP'
            elif isinstance(v, int):
                dtype = 'INTEGER NOT NULL DEFAULT %d' % v
            elif isinstance(v, str):
                dtype = 'VARCHAR(255) NOT NULL DEFAULT \'%s\'' % v
            elif hasattr(v, '_table'):
                dtype = 'INTEGER, FOREIGN KEY (%s) REFERENCES %s(ID)' % (k, v.__name__)
            elif v is None:
                pass
            else:
                raise TypeError("not supported Sql Type %s" % str(v))
        
            operator = 'ADD'
            if k in old_columns:
                operator = 'ALTER COLUMN'

            if v is None:
                operator = 'DROP COLUMN'
            
            if self.Type == 'sqlite' and  v == None:
                # just for sqlite
                drop_columns.add(k)
                old_columns.remove(k)
                sqlitedrop = True
            else:
                cmd = '''ALTER  TABLE {table}
            {operator} {column} {dtype}\n\t;
                            
            '''.format(table=table, operator=operator, column=k, dtype=dtype)
            
            if cmd:
                self.run_cmds(cmd)
            
        
        if sqlitedrop:
            create_sql = self.first('sqlite_master', 'sql', name=table)[0].split(",")
            last_change = False
            last = first_word(create_sql[-1])
            new_create = ''
            delete = set()
            for col in create_sql:
                for m in drop_columns:
                    if m == first_word(col):
                        delete.add(col)
                        if m == last:
                            last_change = True
                        break

            for i in delete:
                create_sql.remove(i)

            cmd = '''ALTER TABLE {table} RENAME TO change_table_tmp_name_this_table_could_not_be_create_by_another_way;'''.format(table=table)
            self.run_cmds(cmd)
            cmd = ','.join(create_sql) if not last_change else ','.join(create_sql) + ");"
            self.run_cmds(cmd)
            cmd = 'INSERT INTO {table} ({values}) SELECT {values} FROM change_table_tmp_name_this_table_could_not_be_create_by_another_way;'.format(table=table, values=','.join(old_columns))
            self.run_cmds(cmd)
            cmd = '''DROP TABLE change_table_tmp_name_this_table_could_not_be_create_by_another_way;'''
            self.run_cmds(cmd)
        
        return True

    def run_cmd(self, cmd, *values):
        try:
            if values:
                self.cu.execute(cmd, values)
            else:
                self.cu.execute(cmd)
        except Exception as e:
            if hasattr(self.con, 'rollback'):
                self.con.rollback()
            print(e)
            print(values)
            print(cmd)
            raise e

        return self.con.commit()

    def run_cmds(self, cmd):
        try:
            self.cu.execute(cmd)
        except Exception as e:
            if hasattr(self.con, 'rollback'):
                self.con.rollback()
            print(e)
            print(cmd)
            raise e

        return self.con.commit()
    # def gen_table(self, *columns):
        # create_cmd = 'create '


class Table:
    """
    this is sql Table trans to class
    need set_handle(objhandle)
    """
    _table = 0
    _obj_handler = None

    def __init__(self, handle=None, **kargs):
        # setattr(self.__class__, '_table', self.__class__.__name__)
        self.ID = None
        self.CreatedTime = None
        self._update = {}

        self._table = self.__class__.__name__
        # for k in self._columns():
        #     if k in kargs:
        #         setattr(self, k, kargs[k])
        #     else:
        #         setattr(self, k, self.__class__.__dict__[k])
        if Table._obj_handler is None:
            if handle:
                Table._obj_handler = handle


        for k in kargs:
            if k in self._columns():
                setattr(self, k, kargs[k])
            elif k in ('ID', 'CreatedTime'):
                setattr(self, k, kargs[k])
            else:
                raise TypeError("table %s no such columns %s " %(self._table, k) )

        # for k in self._obj_columns():
        #     Obj = getattr(self.__class__, k)
        #     v = getattr(self, k)
        #     setattr(self, k, self._obj.find_one(Obj, ID=v))

    def __repr__(self):
        return self.__class__.__name__

    @classmethod
    def _set_handle(cls, handler):
        cls._obj_handler = handler

    @classmethod
    def _columns(cls):
        return [i for i in cls.__dict__ if not i.startswith('__')]

    @classmethod
    def _obj_columns(cls):
        return [i for i in cls._columns() if hasattr(getattr(cls, i), '_table') ]

    def _dict(self):
        return {i:self[i] for i in self.__class__.__dict__ if not i.startswith('__')}

    @classmethod
    def _CREATE_ITEMS(cls):
        return {i:cls.__dict__[i] for i in cls.__dict__ if not i.startswith('__')}

    def __getitem__(self, k):
        v = getattr(self, k)
        if hasattr(getattr(self.__class__, k), '_table'):
            vv = getattr(self.__class__, k)
            return self._obj_handler.find_one(vv, ID=v)
        return v

    def __setitem__(self, k, v):
        self._update[k] = v

    def _save(self):
        if self.ID is None:
            return self._obj_handler.add(self)
        return self._obj_handler.save(self)

    def _delete(self):
        return self._obj_handler.delete(self)

    @staticmethod
    def hash( obj):
        return _hash(obj)

    @classmethod
    def _all(cls):
        return list(cls._obj_handler.find(cls))



class SqlObjectEngine:

    def __init__(self, **connect_option):
        self.sql = SqlEngine(**connect_option)

    def create(self, Obj):
        columns = Obj._CREATE_ITEMS()
        self.sql.create(Obj.__name__, **columns)

    def add(self, Obj_self):
        keys = []
        values = []
        _d = Obj_self._dict()
        for i in _d:
            # jump id and createdTime
            if not _d[i]:
                continue
            keys.append(i)
            values.append(_d[i])

        self.sql.insert(Obj_self.__class__.__name__, keys, *values)

    def _find(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        selected = ','.join(columns)
        return self.sql.select(obj.__name__, selected, **condition)

    def find_one(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        for row in self._find(obj, **condition):
            return obj(**dict(zip(columns, row)))
            

    def find(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        for row in self._find(obj, **condition):
            yield obj(**dict(zip(columns, row)))

    def save(self, obj_self):
        columns = ['ID','CreatedTime'] +obj_self._columns()
        if obj_self.ID != None:
            self.sql.update(obj_self._table, obj_self._update, ID=obj_self.ID)
            obj_self._update = {}

    def delete(self, obj_self):
        self.sql.delete(obj_self._table, ID=obj_self.ID)

    def datetime(self, timeobj):
        if isinstance(timeobj, datetime.datetime):
            return timeobj
        elif isinstance(timeobj, str):
            ss = time.strptime(timeobj, '%Y-%m-%d %H:%M:%S')
            return datetime.datetime.fromtimestamp(time.mktime(ss))





#!/usr/bin/env python3
import requests
import logging
import time
from hashlib import md5
from urllib.parse import quote

logging.level = 1

def check_time(func):
    sta = time.time()
    def _ch(*args, **kargs):
        res = func(*args, **kargs)
        logging.info(time.time() - sta)
        return res
    return _ch

@check_time
def _hash_point(url, additional):
    url = url + quote(additional)
    return md5(requests.get(url).content).hexdigest()



def check_injecable(url, checktor):
    for method in checktor:
        logging.warn("check:" + method)
        judge = checktor[method]
        args  = [_hash_point(url, sql) for sql in method.split("---")]
        logging.warn("hash ok : " + url)
        if judge(*args):
            return True
    return False






def string2bit(string, charset='utf8'):
    """
    translate string to bit list
    """
    s = string.encode(charset)
    for by in s:
        for i in range(8):
           yield  (by >> (7 - i)) & 0x01


def bit2bytes(bits):
    
    count = 0
    b = []
    tmp_b = 0
    for bit in bits:
        count += 1
        tmp_b += bit * (1 << ( 8 - count))
        if count == 8:
            count = 0
            b.append(tmp_b)
            tmp_b = 0
    return bytes(b)




from setuptools import setup, find_packages


setup(name='Mroylib',
    version='1.2.3',
    description='a steady version',
    url='https://git.oschina.net/dark.h/Mroylib.git',
    author='Qing luan',
    author_email='darkhackdevil@gmail.com',
    license='MIT',
    zip_safe=False,
    packages=find_packages(),
    install_requires=['Qtornado','redis','pymysql','fabric3','requests','lxml','bs4','termcolor','bson','simplejson', 'pysocks'],

)





import os, sys, time, datetime
import json
from queue import Queue, Empty
from termcolor import cprint
from urllib.parse import urljoin
from functools import wraps
from collections import defaultdict

sys.path.append(os.getenv("Q_PATH"))
from qlib.asyn import Exe, futures
from qlib.net import to
from qlib.graphy import random_choice
from qlib.spide.agents import AGS
from qlib.spide.web import Handler
from qlib.log import LogControl
from qlib.data.sql import SqlEngine

LogControl.LOG_LEVEL = LogControl.OK
LogControl.LOG_LEVEL |= LogControl.WRN
LogControl.LOG_LEVEL |= LogControl.INFO
LogControl.LOG_LEVEL |= LogControl.FAIL

info = LogControl.info
err = LogControl.err
fail = LogControl.fail
ok = LogControl.ok
wrn = LogControl.wrn


class SqlHandler(Handler):

    def __init__(self, depth=1, *args, **kargs):
        super(SqlHandler, self).__init__(*args, **kargs)
        self.depth = depth
        self.no_depth = 0
        self.sql_filter = Queue()

    def on_start(self, url, parser):
        self.count = 0
        if parser is None:
            return

        for i in parser.xpath("//a[@href!='']"):
            self.ready_link(i.attrib['href'])

        self.run(self.on_detail, limit_wait=self.try_times)

    def on_detail(self, url, parser):
        # LogControl.ok(url)
        # info("parent: ", url, txt_attr=['bold'])
        if parser is None:
            return

        for i in parser.xpath("//a[@href!='']"):
            link = i.attrib['href']
            if self.no_depth < self.depth:
                # info("put", link)
                self.ready_link(link)
                if self.filter(link):
                    ok("find", link)
                    self.sql_filter.put(link)
            else:
                self.dead()

            if link not in self.reach_url:
                ok('  [%d]' % self.count,end='\r')
                sys.stdout.flush()
                self.count += 1
        self.no_depth += 1


class SqlmapApi:
    """
    first use need add 'init' kargs
    like : 
        sqlmapapi(type='mysql', user='root', init=True) #use mysql 
        sqlmapapi(database='/tmp/test.sql', init=True) # use sqlite3
        sqlmapapi(id='cdffe123fab3', database='/tmp/test.sql') # use sqlite3 and load a exist id
        sqlmapapi(database='/tmp/test.sql') # use sqlite3 and create a task
        sqlmapapi(database='/tmp/test.sql', load=True) # use sqlite3 and load the last task id

    Class Task(Table):
        task_id: "some hash id"
        url: str,
        status: 'not running'
        finishtime: time

@get("/task/new")  
@get("/task/<taskid>/delete")  
@get("/admin/<taskid>/list")  
@get("/admin/<taskid>/flush")  
@get("/option/<taskid>/list")  
@post("/option/<taskid>/get")  
@post("/option/<taskid>/set")  
@post("/scan/<taskid>/start")  
@get("/scan/<taskid>/stop")  
@get("/scan/<taskid>/kill")  
@get("/scan/<taskid>/status")  
@get("/scan/<taskid>/data")  
@get("/scan/<taskid>/log/<start>/<end>")  
@get("/scan/<taskid>/log")  
@get("/download/<taskid>/<target>/<filename:path>")  
    """

    TASK_TABLE = {
        'task_id': 'id', 
        'url': 'http://abc.com', 
        'status': 'not running', 
        'data': str, 
        'ftime': time
    }

    TAMPER = [
        'apostrophemask',
        'apostrophenullencode',
        'appendnullbyte',
        'base64encode',
        'between',
        'bluecoat',
        'chardoubleencode',
        'charencode'
    ]

    def __init__(self, id=None, timeout=20, connection="127.0.0.1:8775", load=False, verbose=False, thread_num=4, init=False, **connect_options):
        
        self.exe = Exe(thread_num)
        self.target = 'http://{connection}'.format(connection=connection)
        self.test_url = None
        self.injectable = False
        self.id = id
        self.connect = False
        self.start_time = None
        self.terminated = False
        self.connect_options = connect_options
        self.verbose = verbose
        self.timeout = timeout

        if init:
            db = SqlEngine(**self.connect_options)
            db.create('Task', **self.__class__.TASK_TABLE)

        if self.id is None:
            if load:
                db = SqlEngine(**self.connect_options)
                self.id = db.last('Task', 'task_id')
            else:
                self.create_task()

    def on_result(self, t, v):
        if t == 'new':
            self.id = v['taskid']
            db = SqlEngine(**self.connect_options)
            if db is not None:
                db.insert("Task", ['task_id', 'url', 'status'], self.id, 'None', 'not running')
            db.close()

        elif t == 'data':
            if self.verbose:
                info(v)
            db = SqlEngine(**self.connect_options)
            if v['data']:
                
                if db:
                    upadte_set = {
                        'url': self.test_url,
                        'status': 'injectable',
                        'data': str(v['data']),
                        'ftime': datetime.datetime.now()
                    }
                    db.update("Task", upadte_set, task_id=self.id)
                else:
                    if self.verbose:
                        ok(v['data'])
                    self.injectable = True
            else:
                upadte_set = {
                    'url': self.test_url,
                    'status': 'failed',
                    'ftime': datetime.datetime.now()
                }
                db.update("Task", upadte_set, task_id=self.id)
            db.close()
            # when scan stoped , to delete task;
            self.task_cmd("delete")

        elif t == "status":
            try:
                if self.verbose:
                    info(v[u'status'])
                if v[u'status'] == 'terminated':
                    self.terminated = True
                    self.result()
                else:
                    if not self.terminated:
                        self.exe.timmer(5, self.status)
            except KeyError:
                pass

        elif t == 'start':
            if self.verbose:
                info(v['success'])
        elif t == 'set':
            if self.verbose:
                ok('\ninit options')
        elif t == 'kill':
            if self.verbose:
                fail(v)
        elif t == 'stop':
            if self.verbose:
                wrn(v)
        elif t == 'list':
            for k in v['options']:
                if self.verbose:
                    ok(k, v['options'][k])
        elif t == 'task':
            if self.verbose:
                info(v)
        elif t == 'log':
            for msg in v['log']:
                # if self.log:
                info(msg)

    def handle(self, tag, cmd, **kargs):
        return tag, to(urljoin(self.target, cmd), **kargs).json()

    def create_task(self):
        try:
            self.exe.done(self.handle, self.on_result, 'new', 'task/new')
            self.connect = True
        except Exception as e:
            err("check sqlmapapi server in ")
            self.connect = False

    def delete_task(self):
        self.cmd('task', 'delete')
        self.id = None
        # self.injectable = False
        # self.test_url = None

    def scan_cmd(self, cmd):
        # self.exe.done(self.handle, self.on_result, cmd, 'scan/{id}/{cmd}'.format(id=self.id, cmd=cmd))
        self.cmd('scan', cmd)

    def task_cmd(self, cmd):
        self.cmd('task', cmd)

    def option(self, cmd):
        self.cmd('option', cmd)

    def cmd(self, tag, cmd):
        if self.connect:
            self.exe.done(self.handle, self.on_result, cmd, '{tag}/{id}/{cmd}'.format(tag=tag, id=self.id, cmd=cmd))
        else:
            wrn("not connect")

    def result(self):
        self.scan_cmd("data")

    def scan(self, url, **options):
        '''
        @url: target url
        @options: {
                    'tamper': True,
                    'smart': True,
                    'delay': 1
                }
        '''
        # if 'tamper' in options:
        #     options['tamper'] = SqlmapApi.TAMPER

        self.test_url = url
        if options:
            data = json.dumps(options)
            self.exe.done(
                self.handle,
                self.on_result,
                'set',
                'option/{id}/set'.format(id=self.id),
                data=data,
                method='post',
                headers={'Content-Type': 'application/json'})

        data = json.dumps({'url': url})
        self.exe.done(
            self.handle,
            self.on_result, 
            'start',
            'scan/{id}/start'.format(id=self.id),
            data=data,
            method='post',
            headers={'Content-Type': 'application/json'})

        self.exe.timmer(5, self.status)
        
        self.exe.timmer(self.timeout, self.result)

    def status(self):
        self.scan_cmd('status')


    def log(self):
        self.scan_cmd('log')

    def stop(self, id=None):
        self.scan_cmd("stop")










        
